/// <reference types="node" />
import { Logger } from "./Logger";
import { LogGroupRule } from "./LoggerFactoryService";
import { LogLevel } from "./LoggerOptions";
/**
 * Abstract base logger, extend to easily implement a custom logger that
 * logs wherever you want. You only need to implement doLog(msg: string) and
 * log that somewhere (it will contain format and everything else).
 */
export declare abstract class AbstractLogger implements Logger {
    private open;
    private name;
    private rule;
    private level;
    private _allMessages;
    constructor(name: string, rule: LogGroupRule);
    trace(msg: string, error?: Error | null): void;
    debug(msg: string, error?: Error | null): void;
    info(msg: string, error?: Error | null): void;
    warn(msg: string, error?: Error | null): void;
    error(msg: string, error?: Error | null): void;
    fatal(msg: string, error?: Error | null): void;
    tracec(msg: () => string, error?: () => Error | null): void;
    debugc(msg: () => string, error?: () => Error | null): void;
    infoc(msg: () => string, error?: () => Error | null): void;
    warnc(msg: () => string, error?: () => Error | null): void;
    errorc(msg: () => string, error?: () => Error | null): void;
    fatalc(msg: () => string, error?: () => Error | null): void;
    isTraceEnabled(): boolean;
    isDebugEnabled(): boolean;
    isInfoEnabled(): boolean;
    isWarnEnabled(): boolean;
    isErrorEnabled(): boolean;
    isFatalEnabled(): boolean;
    getLogLevel(): LogLevel;
    isOpen(): boolean;
    close(): void;
    protected abstract doLog(msg: string, logLevel?: LogLevel): void;
    private _log(level, msg, error?);
    private _logc(level, msg, error?);
    private createMessage(level, msg, date, error?);
    private processMessages();
}
/**
 * Simple logger, that logs to the console. If the console is unavailable will throw exception.
 */
export declare class ConsoleLoggerImpl extends AbstractLogger {
    constructor(name: string, rule: LogGroupRule);
    protected doLog(msg: string, logLevel: LogLevel): void;
}
/**
 * Logger which buffers all messages, use with care due to possible high memory footprint.
 * Can be convenient in some cases. Call toString() for full output, or cast to this class
 * and call getMessages() to do something with it yourself.
 */
export declare class MessageBufferLoggerImpl extends AbstractLogger {
    private messages;
    constructor(name: string, rule: LogGroupRule);
    close(): void;
    getMessages(): string[];
    toString(): string;
    protected doLog(msg: string, logLevel: LogLevel): void;
}
